-- Implemented by MrCool (EU, FR, Hyjal, main character: Eilylune)

local data = PVPTooltips_Data;
local patch = PVPTooltips_Patch[GetLocale():sub(1, 2)] or {};

-- We need to do some annoying things to properly handle tooltip modification:
--	  Remove color text from the tooltip (occurs for artifacts)
--    Remove % and thousand separator
--    Preserve decimal separator and replace it by a dot "." so we can parse decimal numbers
--    Calculate the modified value and replace it in the original text by the localized number
--    We also need to handle small values with a decimal part

local TRANSFORM = { };
local KEYS = {"__a__", "__b__", "__c__", "__d__", "__e__"}; -- Should be more than enough

local function Escape(text)
	wipe(TRANSFORM);
	text = text:gsub("|c%x%x%x%x%x%x%x%x.+|r", function(x)
		tinsert(TRANSFORM, x);
		return KEYS[#TRANSFORM];
	end);
	return text;
end

local function Unescape(text)
	for i=1, #TRANSFORM do
		text = text:gsub(KEYS[i], TRANSFORM[i]);
	end
	wipe(TRANSFORM);
	return text;
end

local function GetDecimalSeparator()
	if IsEuropeanNumbers() then
		return "%,", "[ .%%]";
	else
		return "%.", "[ ,%%]";
	end
end

local function WrapColor(text, multiplier)
	if multiplier == 1 then
		return text;
	elseif multiplier < 1 then
		return "|cffff0000" .. text .. "|r";
	else
		return "|cff00ff00" .. text .. "|r";
	end
end

local function ReplaceNumber(x, value)
	local s, e;
	for i=1, #x do
		local code = x:byte(i);
		if code >= 48 and code <= 57 then
			s = s or i;
			e = i;
		end
	end
	if not x:find("%%") then value = math.floor(value + 0.5); end
	return x:sub(1, s-1) .. BreakUpLargeNumbers(value) .. x:sub(e+1);
end

local function TransformValueText(x, multiplier)
	local decimal, pattern = GetDecimalSeparator();
	local value = x:gsub(pattern, ""):gsub(decimal, ".");
	local replacement = ReplaceNumber(x, tonumber(value) * multiplier);
	--print(x, value, replacement);
	return WrapColor(replacement, multiplier);
end

local function ApplyMultiplier(position, multiplier)
	local current = 0
	if type(position) == "table" then
		return function(x)
			current = current + 1
			if tContains(position, current) then
				return TransformValueText(x, multiplier);
			else
				return x
			end
		end
	else
		return function(x)
			current = current + 1
			if current == position then
				return TransformValueText(x, multiplier);
			else
				return x
			end
		end
	end
end

local function PatchTooltip(position, multiplier)
	for i=1, select("#", GameTooltip:GetRegions()) do
		local region = select(i, GameTooltip:GetRegions());
		if region and region:GetObjectType() == "FontString" then
			local text = region:GetText();
			local r, g, b = region:GetTextColor();
			
			-- Identify spell description text
			if text and abs(r - 1.00) < 0.01 and abs(g - 0.82) < 0.01 and abs(b - 0.00) < 0.01 then
				--print(text);
				text = Escape(text);
				text = text:gsub("(%d[%d ,.]*%%?)", ApplyMultiplier(position, multiplier));
				text = Unescape(text);
				region:SetText(text);
			end
		end
	end
end

local function UpdateTooltip(id)
	-- If spell is unspecified, use current spell on tooltip
	if not id then
		local _, _, spellId = GameTooltip:GetSpell();
		id = spellId;
	end

	local info = data[id or 0];
	if info then
		local position = patch[id] or info[1];
		local multiplier = info[2];
		local spec = info[3];
		local details = info[4];
		local index = GetSpecialization();
		local id, name = GetSpecializationInfo(index or 1);

		-- Verify we are in the correct specialization for a PVP modifier to take effect
		if ((not spec) or (id == spec)) and multiplier ~= 1.00 then
			local suffix = "";
			if spec and name then suffix = " (" .. name .. ")"; end
			if details then suffix = suffix .. " (" .. GetSpellInfo(details) .. ")"; end
			
			if UnitIsPVP("player") ~= IsAltKeyDown() then
				PatchTooltip(position, multiplier);
			end
			GameTooltip:AddLine(PVP .. HEADER_COLON .. WrapColor((" %d%%"):format(multiplier * 100), multiplier) .. suffix, 1, 1, 1);
			GameTooltip:AddTexture("Interface\\ICONS\\Ability_DualWield");
			GameTooltip:Show();
		end
	end
end

local function UpdateArtifact(powerID)
	local spellID = C_ArtifactUI.GetPowerInfo(powerID);
	if spellID then
		UpdateTooltip(spellID);
	end
end

do
	-- Setup required hooks
	hooksecurefunc(GameTooltip, "SetAction", function() UpdateTooltip(); end);
	hooksecurefunc(GameTooltip, "SetSpellBookItem", function() UpdateTooltip(); end);
	hooksecurefunc(GameTooltip, "SetSpellByID", function() UpdateTooltip(); end);
	hooksecurefunc(GameTooltip, "SetTalent", function() UpdateTooltip(); end);
	hooksecurefunc(GameTooltip, "SetArtifactPowerByID", function(self, powerID) UpdateArtifact(powerID); end);
end